'''
Created on May 4, 2018

@author: Charlie McLouth
'''
from flask import current_app, g, request
from p4rest.session import P4RestSession, P4RestSessionAdmin, P4RestSessionP4, SESSION_HEADER_NAME
from p4rest.exceptions import P4RestSessionExpiredError, P4RestSessionInvalidError

from flask_restful import abort
from functools import wraps

def require_session(view_function):
    '''decorator to require valid session'''
    @wraps(view_function)
    # the new, post-decoration function. Note *args and **kwargs here.
    def decorated_function(*args, **kwargs):
        #current_app.logger.debug("require_session enter")
        if request.headers.get('Authorization', None):
            #current_app.logger.debug("Authorization header found.")
            try:
                # get the clientip and publickey from headers
                clientip = request.headers.get('X-Forwarded-For', 
                                                  request.remote_addr)
                if clientip is not None:
                    clientip = clientip.split(",")[0].strip()
                authheader = request.headers.get('Authorization', None)
                if authheader is None or len(authheader) < 1:
                    raise P4RestSessionInvalidError("invalid session")
                publickey = None
                for token in authheader.split(","):
                    tokens = token.strip().split("=", 1)
                    if len(tokens) < 2:
                        continue
                    if tokens[0] == SESSION_HEADER_NAME:
                        publickey = tokens[1]
                        break
                kwargs["current_session"] = P4RestSession.createFromKey(clientip, 
                                    publickey, 
                                    current_app.config.get("WORKDIR", None), 
                                    current_app.config.get("P4CONFIG", None))
                # add to global so that it can be accessed in app.teardown_appcontext even if it is expired
                g.current_session = kwargs["current_session"]
                if g.current_session.isExpired():
                    message = "expired session"
                    #current_app.logger.debug(message)
                    raise P4RestSessionExpiredError(message)
                return view_function(*args, **kwargs)
            except:
                #current_app.logger.exception("exception", exc_info=True)
                #current_app.logger.debug("require_session exit")
                abort(401)
        else:
            #current_app.logger.debug("Authorization header not found.")
            #current_app.logger.debug("require_session exit")
            abort(401)
    #current_app.logger.debug("require_session exit")
    return decorated_function

def require_p4session(view_function):
    '''decorator to require p4 session'''
    @wraps(view_function)
    @require_session
    def decorated_function(*args, **kwargs):
        #current_app.logger.debug("require_p4session enter")
        session = kwargs.get("current_session", None)
        if session is not None and isinstance(session, P4RestSessionP4):
            #current_app.logger.debug("require_p4session exit")
            return view_function(*args, **kwargs)
        else:
            #current_app.logger.debug("require_p4session exit")
            abort(400)
    return decorated_function

def require_adminsession(view_function):
    '''decorator to require admin session'''
    @wraps(view_function)
    @require_session
    def decorated_function(*args, **kwargs):
        #current_app.logger.debug("require_adminsession enter")
        session = kwargs.get("current_session", None)
        if session is not None and isinstance(session, P4RestSessionAdmin):
            #current_app.logger.debug("require_adminsession exit")
            return view_function(*args, **kwargs)
        else:
            #current_app.logger.debug("require_adminsession exit")
            abort(400)
    return decorated_function

def require_auth(view_function):
    '''decorator to require valid session'''
    @wraps(view_function)
    # the new, post-decoration function. Note *args and **kwargs here.
    def decorated_function(*args, **kwargs):
        #current_app.logger.debug("require_auth enter")
        P4USER = P4PASSWORD = P4PORT = None
        if "P4PORT" in request.args:
            P4PORT = request.args["P4PORT"]
        if "P4USER" in request.args:
            P4USER = request.args["P4USER"]
        if "P4PASSWORD" in request.args:
            P4PASSWORD = request.args["P4PASSWORD"]
        if request.is_json:
            data = request.get_json()
            P4PORT = data.get("P4PORT", P4PORT)
            P4USER = data.get("P4USER", P4USER)
            P4PASSWORD = data.get("P4PASSWORD", P4PASSWORD)
        if request.authorization is not None \
                and request.authorization.type is not None \
                and request.authorization.type == "basic":
            P4USER = request.authorization.username
            P4PASSWORD = request.authorization.password
        if P4PORT is None or P4USER is None or P4PASSWORD is None:
            #current_app.logger.debug("P4PORT={!s};P4USER={!s}".format(P4PORT, P4USER))
            #current_app.logger.debug("require_auth exit")
            abort(401)
        kwargs["P4PORT"] = P4PORT
        kwargs["P4USER"] = P4USER
        kwargs["P4PASSWORD"] = P4PASSWORD
        #current_app.logger.debug("P4PORT={!s};P4USER={!s}".format(P4PORT, P4USER))
        #current_app.logger.debug("require_auth exit")
        return view_function(*args, **kwargs)
    return decorated_function

